/* $Id: kgi.h,v 1.13 2000/05/23 18:06:10 becka Exp $
******************************************************************************

   KGIcon - KGI interface definitions

   Copyright (C) 1995-1996	Steffen Seeger	[seeger@ggi-project.org]
   Copyright (C) 1995-1996	Andreas Beck	[becka@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _KGI_KGI_H
#define _KGI_KGI_H

#define	NEED_KGI_PROTOTYPES
#include <kgi/types.h>
#include <kgi/kgi_commands.h>


/* ----------------------------------------------------------------------------
 *	useful macros
 * ----------------------------------------------------------------------------
 */

#ifndef MAINTAINER
#define	MAINTAINER	"ggi-develop@eskimo.com"
#endif

/* ----------------------------------------------------------------------------
**	KGI module (display driver)
** ----------------------------------------------------------------------------
*/

#define	KGI_COMMONVERSION	02

enum kgi_flags
{
	FL_MONITOR	= 0x000000FF,
	FL_MON_NORMAL	  = 0x00000000,	/* normal display	*/
	FL_MON_RIGHT_EYE  = 0x00000001,	/* stereo right screen	*/
	FL_MON_LEFT_EYE   = 0x00000002,	/* stereo left screen	*/
	FL_MON_STEREO     = 0x00000003,	/* is stereo?		*/
	FL_MON_INTERLACE  = 0x00000004,	/* interlaced operation	*/
	FL_MON_POWERSAVE  = 0x00000008,	/* has powersave option	*/
	FL_MON_PROPSIZE   = 0x00000010,	/* size depends on mode	*/

	FL_DAC		= 0x0000FF00,
	FL_DAC_8BIT	= 0x00000100,	/* 8 or 6 bit dac	*/
	FL_DAC_2XCLOCK	= 0x00000200,	/* internal 2*clk	*/
	FL_DAC_GAMMA	= 0x00000400,	/* true color LUT	*/

	FL_CHIPSET	   = 0x00FF0000,
	FL_CHIP_LMAP	   = 0x00010000,/* linear mapping	*/
	FL_CHIP_DOUBLESCAN = 0x00020000,
	FL_CHIP_ACCEL	   = 0x00040000,
	FL_CHIP_COLORTEXT  = 0x00080000,

	FL_CLOCK	= 0x7F000000
};

#define	KGI_COMMON \
	char manufact[64],		/* manufacturer		*/\
	model[64];			/* model/trademark	*/\
	kgi_graphmask	modes;		/* supported modes	*/\
	enum kgi_flags	flags;		/* special capabilities	*/\
	kgi_coord	maxdots;	/* maximum resolution	*/\
	void *		priv;		/* private data hook	*/

struct kgi_driver {
	char manufact[64],	/* manufacturer         */
	model[64];		/* model/trademark      */
	int version;		/* version		*/
};

struct	kgi_common
{
	KGI_COMMON
};

#define	MONITORVERSION	02

enum	kgi_display_type
{
	MONO_DISPLAY	= 0x00000000,	/* b&w or grayscale	*/
	HERCMONO,		/* good old Hercules	*/
	LCD,			/* mono LCD		*/
	VGA_MONO,		/* mono VGA monitor	*/

	RGB_DISPLAY	= 0x20000000,	/* RGB color display	*/
	EGA_8,			/* EGA (8 colors)	*/
	EGA_16,			/* EGA (16 colors)	*/
	CGA,			/* CGA color monitor	*/
	VGA_COLOR,		/* VGA color monitor	*/
	PLASMA,			/* plasma flat panel???	*/
	TFT,			/* (???)		*/
	COLOR_LCD,		/*Liquid Crystal Display*/

	NON_RGB_DISPLAY	= 0x40000000,	/* e.g. RGY		*/

	DISPLAY_TYPE	= 0x60000000
};

enum	kgi_sync
{
	SYNC_SEPARATE 	= 0x0000,	/* normal separate sync	*/
	SYNC_ON_RED     = 0x0001,	/* sync on r line	*/
	SYNC_ON_GREEN   = 0x0002,	/* sync on g line	*/
	SYNC_ON_BLUE    = 0x0004,	/* sync on b line	*/
	SYNC_SERRATE	= 0x0008,	/* do hsync during vsync*/
	SYNC_COMPOSITE	= 0x0010,	/* digital composite	*/
	SYNC_TYPEMASK	= 0x001F,
	SYNC_NORMAL	= SYNC_SERRATE,

	SYNC_PEDESTRAL	= 0x0020,	/* need blank pedestral	*/
	SYNC_0700_0300	= 0x0000,	/* <white>_<sync> level	*/
	SYNC_0714_0286	= 0x0040,
	SYNC_0100_0400	= 0x0080,
	SYNC_UNKNOWNLEVEL=0x00C0,	/* not defined by VESA!	*/
	SYNC_SIGNALLEVEL= 0x00C0,	/* mask for signal level*/
		
	SYNC_STANDBY	= 0x0100,	/* standby mode allowed	*/
	SYNC_SUSPEND	= 0x0200,	/* suspend mode allowed */
	SYNC_OFF	= 0x0400,	/* switch off allowed	*/
	SYNC_VESA_DPMS	= 0x0700,	/* has (full) VESA-DPMS	*/

	SYNC_SSYNC_RIGHT= 0x0000,	/* right image on SSYNC	*/
	SYNC_SSYNC_LEFT	= 0x1000,	/* left image on SSYNC	*/
	SYNC_MULTISYNC	= 0x2000	/* multisync monitor	*/
};

struct kgi_monitor 
{
	KGI_COMMON

	kgi_coord	size;		/* physical size in mm	*/
	enum kgi_display_type disptype;	/* display type		*/
	enum kgi_sync	synctype;	/* type of syncing	*/
	kgi_range	dclk;		/* bandwidth		*/
	kgi_range	hfreq;		/* horizontal frequency	*/
	kgi_range	vfreq;		/* vertical frequency	*/

	kgi_color	white;		/* white point		*/
	kgi_uint	gamma;		/* gamma value		*/
};

#define	CHIPSETVERSION	01

struct kgi_chipset
{
	KGI_COMMON

	int		ramsize;	/* installed RAM size	*/
	kgi_range	dclk;		/* DCLK range		*/
};

#define RAMDACVERSION	01

enum kgi_dac_type
{
	DAC_8BIT_PB   = 0x01,		/*   8 bit pixel bus	*/
	DAC_16BIT_PB  = 0x02,		/*  16 bit pixel bus	*/
	DAC_24BIT_PB  = 0x04,		/*  24 bit pixel bus	*/
	DAC_32BIT_PB  = 0x08,		/*  32 bit pixel bus	*/
	DAC_64BIT_PB  = 0x10,		/*  64 bit pixel bus	*/
	DAC_128BIT_PB = 0x20,		/* 128 bit pixel bus	*/
	DAC_PB_MASK   = 0x3F,		/* pixel bus width mask	*/
	DAC_VRAM      = 0x40000000	/* VRAM shift clock	*/
};

struct kgi_ramdac
{
	KGI_COMMON

	enum kgi_dac_type dactype;	/* general device type	     */
	kgi_range 	normal;		/* DCLK range (normal mode)  */
	kgi_range 	dblclk;		/* DCLK (internal doubling)  */
	kgi_range	clock[KGIGT_LAST];	/* DCLK ranges		     */
};

#define	CLOCKVERSION	01

#define	MAX_FIXED_CLOCKS	16
#define	MAX_PROG_CLOCKS		8

enum kgi_clock_type
{
	CLK_TYPE_FIXED,	CLK_TYPE_PROG
};

struct kgi_clock
{
	KGI_COMMON

	enum kgi_clock_type	type;
	union
	{
		int		freq[MAX_FIXED_CLOCKS];
		kgi_range	range[MAX_PROG_CLOCKS];
	} clock;
};

#define	MODEVERSION	04

#define POL_HPOS	0x1000
#define POL_HNEG	0
#define POL_VPOS	0x2000
#define POL_VNEG	0

struct kgi_timing
{			/* all parameters are in dots	*/
	kgiu16	width;		/* visible pixels/lines	*/
	kgiu16	blankstart;	/* assertion of BLANK	*/
	kgiu16	syncstart;	/* assertion of SYNC	*/
	kgiu16	syncend;	/* deassertion of SYNC	*/
	kgiu16	blankend;	/* deassertion of BLANK	*/
	kgiu16	total;		/* total pixels/lines	*/
	kgiu16	polarity;	/* sync polarity	*/
};

struct kgi_preset
{
	int	dclk;		/* dot clock in Hz	*/
	struct kgi_timing x,y;	/* x and y timing values*/
};

struct kgi_mode
{			/* mode setup info, calculated or preset*/
	kgi_modereq	request;/* requested mode	*/

	  			/* calculated 'on the fly'	*/
	struct	kgi_timing	x,y;	/* timing parameters	*/
	kgi_coord	magnify;/* stretched ... times	*/
	enum	kgi_flags	flags;	/* special capabilities	*/
	int		dclk;	/* dot (master)		*/
	kgi_ratio	clk;	/* clock rate programmed*/
	kgi_ratio	lclk;	/* transfer rate to dac (byte/s)*/

	/* set by kernel driver	*/
	mem_vaddr	fb;	/* fb address		*/
	kgiu32		fb_size;/* total buffer size	*/
	kgiu32		fsize;	/* size of one frame	*/

	struct kgi_fb_ops *io;	/* ops to use when accessing hw */
	struct kgi_fb_ops *mem;	/* ops to use when accessing mem */
	struct kgi_mmio_region *mmio;	/* possible mmio regions */
	struct kgi_accel	*accel;	/* possible accelerators */

/* !!! change them to union driver_info ... */
	int		monitor;/* driver specific,	*/
	int		ramdac;	/* check_timing should	*/
	int		clock;	/* assume them not valid*/
	int		chipset;
/* For KGI mmio accel */   
	unsigned long fbcon_mmio_start;
	unsigned long fbcon_mmio_len;
	unsigned long fbcon_accel;
};

enum kgi_tm_cmd
{
	CMD_PROPOSE,		/* propose a timing	*/
	CMD_LOWER,		/* only lower clock rate*/
	CMD_RAISE,		/* only raise clock rate*/
	CMD_CHECK,		/* don't modify but check*/
	CMD_READY		/* don't care.		*/
};

#define	INFOVERSION	01

union kgi_info		/* union to ensure proper mem allocation  */
{
	struct kgi_common	common;

	struct kgi_monitor	monitor;
	struct kgi_ramdac	ramdac;
	struct kgi_clock	clock;
	struct kgi_chipset	chipset;
};

struct kgi_card
{
	KGI_COMMON
	struct kgi_monitor	*monitor;
	struct kgi_ramdac	*ramdac;
	struct kgi_clock	*clock;
	struct kgi_chipset	*chipset;

	struct kgi_mode		preset;
};

/* ----------------------------------------------------------------------------
 *	display management
 * ----------------------------------------------------------------------------
 *	Some terms:
 *

 *	display	- device that converts pixeldata into realworld images.
 *		- has to be enabled before any access using ENABLE_DISPLAY(dpy);
 *		- has a certain mode that determines how pixeldata is converted
 *		- may have up to eight framebuffers/CLUTs (one or two displayed)
 *		- may have (hardware) pointer/cursor shape.
 *
 *	device	- a combination of a display facilities
 *		- is mapped (displayed) on a display
 *		- may be either a console or a graphic device
 *
 *	console	- special type of device that displays characters
 */

#define display	kgi_display
#define	pixmap	kgi_pixmap
#define	pixel	kgi_pixel
#define	attr	kgi_attr
#define	fb_ops	kgi_fb_ops
#define	FBARGD	pixmap *fb
#define	FBARGC	fb

#include <kgi/pixmap.h>

#undef	display
#undef	pixmap
#undef	pixel
#undef	attr
#undef	fb_ops
#undef	FBARGD
#undef	FBARGC

#define	MAX_NR_DISPLAYS		32
#define	MAX_NR_DEVICES		(2*MAX_NR_CONSOLES)
#define	NR_FB			8	/* (max) frames per display	*/

#define	VALID_DEV_ID(x)		((0 <= x) && (x < MAX_NR_DEVICES))
#define	VALID_DPY_ID(x)		((0 <= x) && (x < MAX_NR_DISPLAYS))

struct kgi_mmio_region 
{
	struct kgi_mmio_region *next;
	char	*name;	/* name of the region		*/
	int	type;	/* standard types		*/
#define MMIO_FRAMEBUFFER 1

	int	prot;   /* mapping protection flags	*/

        kgiu32		size;  	/* total size of the region     */
	kgi_addr	psize;	/* physical size of window	*/
        kgi_addr	pstart; /* physical address of window   */
	kgi_addr	offset;	/* logical address at pstart	*/

	void	(*set_offset)(struct kgi_mmio_region *, kgi_addr offset);
	void	*private;
};

/* !!! fix system dependent data types! */
struct kgi_mmio_mapping
{
	struct kgi_mmio_mapping	*next;	/* next in list			*/

	kgi_uint	devid;		/* device this belongs to	*/
	kgi_uint	mmap;		/* MMAP_LINEAR or MMAP_PAGED	*/
#define	MAP_LINEAR_LINEAR	1
#define	MAP_LINEAR_PAGED	2
#define	MAP_PAGED_LINEAR	3
#define	MAP_PAGED_PAGED		4

	struct kgi_mmio_region *r;	/* the region we are mapping	*/
	struct kgi_mmio_mapping **other;/* pointer to list head		*/
	struct file *file;

	kgi_addr	offset;		/* offset for paged mappings	*/
	struct mm_struct *mm;		/* mapping table struct		*/
	kgi_addr	mstart, mend;	/* start/end of mapped region	*/
};

struct kgi_accel_buffer {

	struct kgi_accel_buffer	*next;	/* next buffer to process	*/
	kgi_uint	state;		/* state			*/
#define	ACCEL_IDLE	0	/* nothing to do	*/
#define	ACCEL_FILL	1	/* gets filled by app	*/
#define	ACCEL_EXEC	2	/* being executed	*/
#define	ACCEL_WAIT	3	/* waiting for exexution*/
	kgi_uint	size;		/* size of buffer		*/
	void	*buffer;		/* page aligned pointer		*/
	void	*alloc;			/* pointer to allocated memory	*/
};

struct kgi_accel {

	struct kgi_accel	*next;	/* next accelerator		*/
	struct kgi_display	*dpy;

	char	*name;			/* identifier			*/
	kgi_uint	type;		/* numeric ID			*/

	void	(*execute)(struct kgi_accel*, struct kgi_accel_buffer *);
	void    (*wait)(struct kgi_accel*);
};

struct kgi_accel_mapping {

	struct kgi_accel_mapping *next;	/* next mapping of this accel	*/
	struct kgi_accel_mapping **other;/*other mappings of this accel	*/
	struct kgi_accel	*accel;	/* the executing accelerator	*/
	
	struct file 	*file;		/* file this is associated to	*/
	kgi_uint	devid;		/* ID of special file		*/

	struct kgi_accel_buffer	ping;	/* command buffer		*/
	struct kgi_accel_buffer	pong;	/* command buffer		*/
};

struct kgi_graphic;

typedef int kgi_command_fn(struct kgi_graphic *, kgi_uint, kgi_uint);

enum kgi_pointer_mode
{

	pmWindows, pmX11, pmThreeColor
};

struct kgi_pointer64x64
{
	enum kgi_pointer_mode	mode;
	kgiu8	xor_mask[512];
	kgiu8	and_mask[512];
	kgi_uint	hotx, hoty;
	kgi_color	col[3];
};

struct kgi_display
{				/* these are set by the kgi kernel code	*/
	int	id;		/* display number, init to -1		*/
	kgi_uint	refcnt;	/* reference count, init to 0		*/
	kgiu32	conflict_text;	/* displays in conflict when in textmode*/
	kgiu32	conflict_graph;	/* displays in conflict when in graphics*/
	kgiu32	conflict_setup;	/* displays in conflict during setup	*/
	struct kgi_device *dev;	/* device currently mapped		*/

				/* these are set by the driver		*/
	void	*driver_data;		/* internal data of the driver	*/

	void	(*enable)(struct kgi_display *);
	void	(*disable)(struct kgi_display *);
	void	(*reset)(struct kgi_display *);

	kgi_command_fn *cmd[KGI_NR_SUBSYSTEMS];	/* subsystem command handlers */
	struct kgi_card		*card;		/* info about the hardware */

	int (*check_mode)(struct kgi_display*,struct kgi_mode*,enum kgi_tm_cmd);

        void (*set_mode)(struct kgi_display *, struct kgi_mode *);
	void (*set_font)(struct kgi_display *, struct kgi_font *, kgi_uint, kgi_uint);
	void (*set_clut)(struct kgi_display *, kgi_clut *, kgi_uint, kgi_uint);
	void (*set_split)(struct kgi_display*, int);
	void (*set_frame)(struct kgi_display*, int);
	void (*set_origin)(struct kgi_display *, kgi_uint, kgi_uint);
	void (*set_pointer_shape)(struct kgi_display*,struct kgi_pointer64x64*);

        void (*show_pointer)(struct kgi_display *, kgi_uint, kgi_uint);
	void (*hide_pointer)(struct kgi_display *);
        void (*undo_pointer)(struct kgi_display *);/*only if show modifies fb */

	void (*show_cursor)(struct kgi_display *, kgi_uint, kgi_uint);
	void (*hide_cursor)(struct kgi_display *);
	void (*undo_cursor)(struct kgi_display *); /*only if show modifies fb */

	struct kgi_mode	mode;		/* current mode of the display	*/

	kgi_modereq	*defmode;	/* default mode for this display*/
	kgi_pixmap	fb;		/* framebuffer of current frame	*/
};

struct kgi_device	/* common fields for console/graphic devices	*/
{
	int	id;
	int	refcnt;

	kgiu16	type;			/* console or graphic device	*/
#define	KGI_CONSOLE_DEV	0
#define	KGI_GRAPHIC_DEV	1

	kgiu16	state;			/* various state control bits	*/
#define	KGI_MAPPED_ON_DISPLAY	1	/* display map state	*/
#define	KGI_DO_BACKINGSTORE	2	/* keep fb when in bg	*/
#define	KGI_POINTER_TO_SHOW	4	/* show pointer		*/
#define	KGI_POINTER_SHOWN	8	/* pointer visible	*/
#define	KGI_POINTER_FB_MODIFIED	16	/* pointer drawn in fb	*/
#define	KGI_CURSOR_TO_SHOW	32	/* show cursor		*/
#define	KGI_CURSOR_SHOWN	64	/* cursor visible	*/
#define	KGI_CURSOR_FB_MODIFIED	128	/* cursor drawn in fb	*/

	/* set by KGI manager when the device is attached to a display */
	struct	kgi_display	*dpy;	/* display used for viewing	*/
	struct	kgi_fb_ops	*mem;	/* framebuffer ops when in bg	*/

		kgi_modereq	mode;	/* display mode to be used	*/
		kgi_uint		frame;	/* frame displayed		*/
		kgi_uint		split;	/* split position for frame 0	*/
		kgi_coord	org[NR_FB]; /* origin *pixel*		*/
		kgi_clut	lut[NR_FB]; /* look up tables		*/
		kgi_pixmap	 fb[NR_FB]; /* frame buffers		*/
	struct	kgi_font	font;	/* font to use for text output	*/
	struct kgi_pointer64x64	*ptr_shape;	/* pointer shape	*/
		kgi_coord	cursor;		/* cursor hot spot	*/
		kgi_coord	pointer;	/* pointer hot spot	*/
		kgi_coord	ptr_max;	/* pointer window max	*/
		kgi_coord	ptr_min;	/* pointer window min	*/

	void (*save)(struct kgi_device *);
	void (*restore)(struct kgi_device *);

	struct	tty_struct	*tty;	/* associated tty		*/
};

#include <kgi/graphic.h>

/*\
|*| ----------------------------------------------------------------------------
|*|	KGI manager interface
|*| ----------------------------------------------------------------------------
\*/

extern struct kgi_display	*kgidpy[MAX_NR_DISPLAYS];
extern struct kgi_device	*kgidev[MAX_NR_DEVICES];

extern void *kgi_alloc_device(kgi_uint num, int priority);
extern void kgi_free_device(struct kgi_device *dev);
#define kgi_device_allocated(num) (device[num])

extern int kgi_attach_device(struct kgi_device *dev);
extern void kgi_detach_device(struct kgi_device *dev);
extern void kgi_map_device(struct kgi_device *dev);
extern void kgi_unmap_device(struct kgi_device *dev);
extern void kgi_show_gadgets(struct kgi_device *dev);
extern void kgi_undo_gadgets(struct kgi_device *dev);
extern void kgi_hide_gadgets(struct kgi_device *dev);

extern int kgi_check_mode(struct kgi_display*,struct kgi_mode*,enum kgi_tm_cmd);
extern void kgi_set_mode(struct kgi_display *, struct kgi_mode *);
extern void kgi_set_pointer(struct kgi_device *dev, kgi_uint x, kgi_uint y);
extern void kgi_set_pointer_window(struct kgi_device *dev, kgi_uint xmin, kgi_uint xmax,
	kgi_uint ymin, kgi_uint ymax);
extern void kgi_set_origin(struct kgi_device *dev, kgi_uint frame, kgi_uint x, kgi_uint y);
extern void kgi_set_split(struct kgi_device *dev, kgi_uint line);

extern void kgi_init(void);
extern kgiu32 kgi_console_init(kgi_addr kmem_start, kgi_addr kmem_end);
extern kgiu32 kgi_boot_init(kgi_addr kmem_start, kgi_addr kmem_end);
extern int display_init(void);

extern int kgi_register_display(struct kgi_display *dpy, int id);
extern int kgi_unregister_display(struct kgi_display *dpy, struct kgi_display *alias);
extern void kgi_enable_display(struct kgi_display *dpy, int setup);

extern kgi_clut *kgi_default_clut(kgi_modereq *mode);

extern kgi_fb_ops	text16_mono_io, text16_mono_mem;
extern kgi_fb_ops	text16_color_io, text16_color_mem;

extern struct kgi_display *text16_malloc_display(size_t fb, kgi_uint x, kgi_uint y,
	kgi_uint textx, kgi_uint texty, int color);
extern void text16_free_display(struct kgi_display *dpy);

#endif	/* #ifdef _KGI_KGI_H */
