/*******************************************************************************
 * Western Digital (Paradise) common chipset family structures and functions
 *******************************************************************************
 *
 *   Copyright (C) 1997 by Brian S. Julin <bri@calyx.com>
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************

 * Version 1.1 Brian S. Julin
 * Major: Copied IBM vga driver from GGI 0.0.9 11/20/97 Snapshot
 *
 *******************************************************************************
 * 
 * This code was derived from the following sources of information:
 * [1] VGADOC Version 4b Finn Thoegersen
 */

#ifndef _wd_family_h
#define _wd_family_h

#include <WD/temporary_hacks.h>
/* #include <kgi/kgi.h> */

struct kgi_wd_chipset {
  struct kgi_chipset chipset;
  int chiptype;                          /* Model of chip       */
  int features;                          /* active features     */
  char revstring[15];  /* Revision string */
#define BANKING_RAM 0x0001
};


#define CHIPSET_NOT_WD   (0x00)
#define CHIPSET_PVGA1A   (0x01)
#define CHIPSET_WD90C90  (0x02)
#define CHIPSET_PVGA1B   (0x03)
#define CHIPSET_WD90C00  (0x03)  /* Same thing */
#define CHIPSET_WD90C10  (0x04)
#define CHIPSET_WD90C11  (0x05)
#define CHIPSET_WD90C20  (0x06)
#define CHIPSET_WD90C20A (0x07)
#define CHIPSET_WD90C22  (0x08)
#define CHIPSET_WD90C24  (0x09)
#define CHIPSET_WD90C24A (0x0A)
#define CHIPSET_WD90C26A (0x0B)
#define CHIPSET_WD90C30  (0x0C)
#define CHIPSET_WD90C31  (0x0D)
#define CHIPSET_WD90C33  (0x0E)
#define CHIPSET_UNKNOWN_WD (0x0F)

#define WD_HOWMANY_CHIPSETS CHIPSET_UNKNOWN_WD + 1

/* Keep these here since they are bulky so as not to force inclusion */
#define WD_VERBOSE_NAMES \
  "not WD chipset", "PVGA1A", "wd90c90", "PVGA1B or wd90c00",\
  "wd90c10", "wd90c11", "wd90c20", "wd90c20a", "wd90c22",\
  "wd90c24", "wd90c24a", "wd90c26a", "wd90c30", "wd90c31",\
  "wd90c33", "Unknown wd9xxxx"

#define WDFOUND(X) {}

/* Decide whether we are compiling, or just including, this library */
#ifdef _wd_family_c
int kgi_chipset_wd_detect(struct kgi_wd_chipset *);
void kgi_chipset_wd_findram(struct kgi_wd_chipset *);
#else
extern int kgi_chipset_wd_detect(struct kgi_wd_chipset *);
extern void kgi_chipset_wd_findram(struct kgi_wd_chipset *);
#endif

#endif /* not _wd_family_h */

